/******************************************************************************
 *
 *	Multi-Channel USB audio (SINICHIRO) library header : sinich_lib.h
 *
 *	Copyright (c) 2006 by TED 
 *
 *
 *	NOTES:
 *	-
 */
#ifndef __SINICH_LIB_H
#define __SINICH_LIB_H


/******************************************************************************
 *	include files
 */


/******************************************************************************
 *	definitions
 */
// SINICHIRO I/O data type definition
#define	HANDLE			unsigned long int

#ifdef _LINUX_DEF_ // define check #0
#define TDBDUSBIF_API

#else // define check #0
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <winioctl.h>
#include <winbase.h>

#ifdef TDBDUSBIF_EXPORTS
#define TDBDUSBIF_API __declspec(dllexport)
#else
#define TDBDUSBIF_API __declspec(dllimport)
#endif

// This class is exported from "tdbdusbif.dll"
class TDBDUSBIF_API CTdbdusbif {
public:
	CTdbdusbif(void);
	// TODO: Add method in this position
};
#endif // define check #0

/* data endian set */
#define SINICH_BIG_ENDIAN			0x20
#define SINICH_LITTLE_ENDIAN		0x00

/* audio sampling definition */
#define AUDIO_48K				0x00
#define AUDIO_32K				0x01
#define AUDIO_16K				0x02
#define AUDIO_8K				0x03
#define AUDIO_44K				0x04
#define AUDIO_22K				0x05
#define AUDIO_11K				0x06

/* audio channel definition */
#define AUDIO_CHEN_MASK			0x18
#define AUDIO_CHEN16			0x00
#define AUDIO_CHEN32			0x08
#define AUDIO_CHEN48			0x10
#define AUDIO_CHEN64			0x18


// May 20, 2005 : modify
// ICHIRO I/O data structure
struct ich_io_t {
	uchar			dir;			// data stream
	HANDLE			dev;			// SINICHIRO device handler
};


struct codec_param_t {
	uchar			resv1;
	uchar			adc_power_down;
	uchar			audio_led;
	uchar			input_select;
	uchar			pwr_management;
	uchar			mode_control;
	uchar			timer_select;
	uchar			alc_mode_control1;
	uchar			alc_mode_control2;
	uchar			operation_mode;
	uchar			input_pga_control;
	uchar			peak_hold_even_ch;
	uchar			peak_hold_odd_ch;
	uchar			resv2;
	uchar			resv3;
	uchar			resv4;
	ushort			act_flag;
};


struct io_param_t {
	uchar			resv1;
	uchar			led;
	uchar			gpio_dir0;
	uchar			gpio_dir1;
	uchar			gpio_dir2;
	uchar			pwm_ctrl;
	uchar			pwm_logic0;
	uchar			pwm_logic1;
	uchar			pwm_logic2;
	uchar			pwm_logic3;
	uchar			pwm_logic4;
	uchar			pwm_logic5;
	uchar			pwm_logic6;
	uchar			pwm_logic7;
	uchar			resv2;
	uchar			adc_range;
	ushort			act_flag;
};


struct act_info_t {
	uchar			codec0_act_info;
	uchar			codec1_act_info;
	uchar			codec2_act_info;
	uchar			codec3_act_info;
	uchar			io4_act_info;
	uchar			io5_act_info;
	uchar			io6_act_info;
	uchar			io7_act_info;
};


typedef struct _USB_DEVICE_DESCRIPTOR {
    UCHAR bLength;
    UCHAR bDescriptorType;
    USHORT bcdUSB;
    UCHAR bDeviceClass;
    UCHAR bDeviceSubClass;
    UCHAR bDeviceProtocol;
    UCHAR bMaxPacketSize0;
    USHORT idVendor;
    USHORT idProduct;
    USHORT bcdDevice;
    UCHAR iManufacturer;
    UCHAR iProduct;
    UCHAR iSerialNumber;
    UCHAR bNumConfigurations;
} USB_DEVICE_DESCRIPTOR, *PUSB_DEVICE_DESCRIPTOR;

#ifdef _LINUX_DEF_ // define check #2
#else // define check #2

struct _USB_ENDPOINT_DESCRIPTOR {
    UCHAR bLength;
    UCHAR bDescriptorType;
    UCHAR bEndpointAddress;
    UCHAR bmAttributes;
    USHORT wMaxPacketSize;
    UCHAR bInterval;
} USB_ENDPOINT_DESCRIPTOR, *PUSB_ENDPOINT_DESCRIPTOR;

struct _USB_CONFIGURATION_DESCRIPTOR {
    UCHAR bLength;
    UCHAR bDescriptorType;
    USHORT wTotalLength;
    UCHAR bNumInterfaces;
    UCHAR bConfigurationValue;
    UCHAR iConfiguration;
    UCHAR bmAttributes;
    UCHAR MaxPower;
} USB_CONFIGURATION_DESCRIPTOR, *PUSB_CONFIGURATION_DESCRIPTOR;

struct _USB_INTERFACE_DESCRIPTOR {
    UCHAR bLength;
    UCHAR bDescriptorType;
    UCHAR bInterfaceNumber;
    UCHAR bAlternateSetting;
    UCHAR bNumEndpoints;
    UCHAR bInterfaceClass;
    UCHAR bInterfaceSubClass;
    UCHAR bInterfaceProtocol;
    UCHAR iInterface;
} USB_INTERFACE_DESCRIPTOR, *PUSB_INTERFACE_DESCRIPTOR;
#endif // define check #2


/******************************************************************************
 *	SINICHIRO API
 */
TDBDUSBIF_API int sinich_sys_open(struct ich_io_t **io, uchar *name);
TDBDUSBIF_API int sinich_sys_close(struct ich_io_t *io);
TDBDUSBIF_API int sinich_sys_set_reg(struct ich_io_t *io, uchar reg, uchar ctl);
TDBDUSBIF_API int sinich_sys_get_reg(struct ich_io_t *io, uchar reg, uchar *ctl);
//TDBDUSBIF_API int sinich_sys_set_allreg(struct ich_io_t *io, ushort reg, uchar ctl);		// Generally no support 2006/05/01
//TDBDUSBIF_API int sinich_sys_get_allreg(struct ich_io_t *io, ushort reg, uchar *ctl);	// Generally no support 2006/05/01
TDBDUSBIF_API int sinich_sys_act_info(struct ich_io_t *io, uchar board, uchar *info);
TDBDUSBIF_API int sinich_sys_start(struct ich_io_t *io);
TDBDUSBIF_API int sinich_sys_stop(struct ich_io_t *io);
TDBDUSBIF_API ushort sinich_sys_status(struct ich_io_t *io);
TDBDUSBIF_API int sinich_pcm_ctl(struct ich_io_t *io, uchar ctl);
TDBDUSBIF_API int sinich_pcm_set_codec(struct ich_io_t *io, uchar board, struct codec_param_t *param);
TDBDUSBIF_API int sinich_pcm_get_codec(struct ich_io_t *io, uchar act, struct codec_param_t *param);
TDBDUSBIF_API int sinich_pcm_read(struct ich_io_t *io, uchar *buffer, ulong size);
TDBDUSBIF_API int sinich_pcm_write(struct ich_io_t *io, uchar *buffer, ulong size);
//TDBDUSBIF_API int sinich_sys_get_codec_ctl(struct ich_io_t *io, uchar board_sel, uchar codec_sel, uchar codec_addr, uchar *codec_data);
//TDBDUSBIF_API int sinich_sys_set_codec_ctl(struct ich_io_t *io, uchar board_sel, uchar codec_sel, uchar codec_addr, uchar codec_data);
TDBDUSBIF_API int sinich_sys_get_fpga_version(struct ich_io_t *io);
TDBDUSBIF_API int sinich_sys_get_driver_version(struct ich_io_t *io);
TDBDUSBIF_API int get_device_descriptor(HANDLE hdev, PUSB_DEVICE_DESCRIPTOR dsc);
TDBDUSBIF_API int sinich_all_start(struct ich_io_t *io);
TDBDUSBIF_API int sinich_all_stop(struct ich_io_t *io);

#endif /* __SINICH_LIB_H */
