/******************************************************************************
 *
 *	Multi-Channel USB audio (SINICHIRO) driver header : sinich_drv.h
 *
 *	Copyright (c) 2006 by TED 
 *
 *
 *	NOTES:
 *	-
 */
#ifndef __SINICH_DRV_H
#define __SINICH_DRV_H


/******************************************************************************
 *	include files
 */
#include <linux/version.h>


/******************************************************************************
 *	definitions
 */
// Maxmum number of SINICHIRO card
#define SNDRV_CARDS	1

// URB active flag
#define SINICH_URB_IS_NOT_ACTIVE			0
#define SINICH_URB_IS_ACTIVE			1

// Maximum number of URB
#define SINICH_EP04_MAX_URB			8
#define SINICH_EP08_MAX_URB			1

// driver version number
#define SINICH_DRIVER_VERSION			0x0100

#define RNGBUF_NUM 32
#define RNGBUF_MSK 0x01F


// SINICHIRO chip instance structure
struct snd_SINICHIRO
{
	int			index;
	struct usb_device	*dev;
	int			shutdown;
	int			num_interfaces;

	struct urb		*IntIn01urb;				// endpoint 1 URB structure pointer
	struct urb		*BlkIn02urb[2];				// endpoint 2 URB structure pointer
	struct urb		*BlkOut04urb;				// endpoint 4 URB structure pointer

	wait_queue_head_t	IntIn01EndNotify;		// endpoint 1 wait queue
	wait_queue_head_t	IntIn01TickNotify;		// endpoint 1 wait queue

	// additional member of new sinich
	ushort			pcm_status;					// read fpga fifo status
	uchar			chain_status;				// interrupt chain status;
	ushort			str_status;					// stored status(copy pcm_status)
	spinlock_t		sts_lock;					// 

	uchar			vendorBuf[64];
	//
	uchar			*rcvBuf[RNGBUF_NUM];			// ring Buffer for Bulkin
	uchar			*sndBuf[RNGBUF_NUM];			// ring Buffer for Bulkout
	uchar			rcvIndex;					// rcvd ring Buffer Index
	uchar			sndIndex;					// snd ring Buffer Index
	uchar			rcvCount;					// rcv ring Buffer Valid-Data-Blk Count
	uchar			wrCount;					// snd ring Buffer Valid-Data-Blk Count
	uchar			rdIndex;					// rcv ring Buffer read Index
	uchar			wrIndex;					// snd ring Buffer write Index
	ushort			inpkt_size;					// blkin pkt size
	ushort			inpkt_size4;					// blkin pkt size
	ushort			outpkt_size;				// blkout pkt size
	uchar			tgl;						//
	uchar			inpass;
	uchar			outpass;
	uchar			snd;

	uchar			isopen;				// device open counter
};

typedef struct snd_SINICHIRO snd_SINICHIRO_t;


/******************************************************************************
 *	macros
 */
#ifndef get_iface_desc
#define get_iface_desc(iface)	(&(iface)->desc)
#define get_endpoint(alt,ep)	(&(alt)->endpoint[ep].desc)
#define get_ep_desc(ep)		(&(ep)->desc)
#define get_cfg_desc(cfg)	(&(cfg)->desc)
#endif

#ifndef usb_pipe_needs_resubmit
#define usb_pipe_needs_resubmit(pipe) 1
#endif

#ifndef snd_usb_complete_callback
#define snd_usb_complete_callback(x) (x)
#endif

#ifndef snd_usb_get_speed
#define snd_usb_get_speed(dev) ((dev)->speed)
#endif


#endif /* __SINICH_DRV_H */
